﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Services.OKB.DataAccess;

namespace HeuristicLab.Services.OKB {
  public class AlgorithmParameterNameFilter : IFilter {
    public DataTransfer.SetComparison Comparison { get; set; }
    public string Value { get; set; }

    public AlgorithmParameterNameFilter(DataTransfer.SetComparison comparison, string value) {
      Comparison = comparison;
      Value = value;
    }
    public AlgorithmParameterNameFilter(DataTransfer.SetComparisonStringFilter filter) {
      Comparison = filter.Comparison;
      Value = filter.Value;
    }

    public IQueryable<Run> Apply(IQueryable<Run> source) {
      switch (Comparison) {
        case DataTransfer.SetComparison.Equal:
          return source.Where(x => x.Experiment.Algorithm.AlgorithmParameters.Any(y => (y.Name == Value) || (y.Alias == Value)));
        case DataTransfer.SetComparison.NotEqual:
          return source.Where(x => x.Experiment.Algorithm.AlgorithmParameters.All(y => (y.Name != Value) && (y.Alias != Value)));
        case DataTransfer.SetComparison.Contains:
          return source.Where(x => x.Experiment.Algorithm.AlgorithmParameters.Any(y => y.Name.Contains(Value) || y.Alias.Contains(Value)));
        case DataTransfer.SetComparison.NotContains:
          return source.Where(x => x.Experiment.Algorithm.AlgorithmParameters.All(y => !y.Name.Contains(Value) && !y.Alias.Contains(Value)));
        default:
          return source;
      }
    }
  }
}
