﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Services.OKB.DataAccess;

namespace HeuristicLab.Services.OKB {
  public class RunFinishedDateFilter : IFilter {
    public DataTransfer.OrdinalComparison Comparison { get; set; }
    public DateTime Value { get; set; }

    public RunFinishedDateFilter(DataTransfer.OrdinalComparison comparison, DateTime value) {
      Comparison = comparison;
      Value = value;
    }
    public RunFinishedDateFilter(DataTransfer.OrdinalComparisonDateTimeFilter filter) {
      Comparison = filter.Comparison;
      Value = filter.Value;
    }

    public IQueryable<Run> Apply(IQueryable<Run> source) {
      switch (Comparison) {
        case DataTransfer.OrdinalComparison.Less:
          return source.Where(x => x.FinishedDate < Value);
        case DataTransfer.OrdinalComparison.LessOrEqual:
          return source.Where(x => x.FinishedDate <= Value);
        case DataTransfer.OrdinalComparison.Equal:
          return source.Where(x => x.FinishedDate == Value);
        case DataTransfer.OrdinalComparison.GreaterOrEqual:
          return source.Where(x => x.FinishedDate >= Value);
        case DataTransfer.OrdinalComparison.Greater:
          return source.Where(x => x.FinishedDate > Value);
        case DataTransfer.OrdinalComparison.NotEqual:
          return source.Where(x => x.FinishedDate != Value);
        default:
          return source;
      }
    }
  }
}
